@@ -50,6 +50,9 @@ options = [make_option('-m', '--mail',
            make_option('-i', '--ignore',
                        help = 'ignore the applied patches in the series',
                        action = 'store_true'),
+           make_option('--replace',
+                       help = 'replace the unapplied patches in the series',
+                       action = 'store_true'),
            make_option('-b', '--base',
                        help = 'use BASE instead of HEAD for file importing'),
            make_option('-e', '--edit',
@@ -231,6 +234,11 @@ def __import_patch(patch, filename, options):
         message, author_name, author_email, author_date = \
                  __parse_patch(filename)
 
+    if not patch:
+        patch = make_patch_name(message)
+        if not patch:
+            raise CmdException, 'Unknown patch name'
+
     # refresh_patch() will invoke the editor in this case, with correct
     # patch content
     if not message:
@@ -248,6 +256,9 @@ def __import_patch(patch, filename, options):
     if options.commemail:
         committer_email = options.commemail
 
+    if options.replace and patch in crt_series.get_unapplied():
+        crt_series.delete_patch(patch)
+
     crt_series.new_patch(patch, message = message, can_edit = False,
                          author_name = author_name,
                          author_email = author_email,
@@ -317,7 +328,7 @@ def func(parser, options, args):
         elif filename:
             patch = os.path.basename(filename)
         else:
-            raise CmdException, 'Unknown patch name'
+            patch = ''
         if options.strip:
             patch = __strip_patch_name(patch)
 
