@@ -29,9 +29,11 @@ usage = """%prog [options] <name>
 
 Create a new, empty patch and make it the topmost one. If the
 '--message' option is not passed, an editor is invoked with the
-.git/patchdescr.tmpl file used a as template, together with generated
-lines. By default, the local changes in the working tree are not included
-in the patch. A 'refresh' command is needed for this."""
+.git/patchdescr.tmpl, ~/.stgit/templates/patchdescr.tmpl or
+/usr/share/stgit/templates/patchdescr.tmpl file used a as template,
+together with generated lines. By default, the local changes in the
+working tree are not included in the patch. A 'refresh' command is
+needed for this."""
 
 options = [make_option('-m', '--message',
                        help = 'use MESSAGE as the patch description'),
