@@ -37,8 +37,13 @@ options = [make_option('-n', '--name',
            make_option('-r', '--reverse',
                        help = 'reverse the commit object before importing',
                        action = 'store_true'),
+           make_option('-p', '--parent', metavar = 'COMMITID',
+                       help = 'use COMMITID as parent'),
            make_option('--fold',
                        help = 'fold the commit object into the current patch',
+                       action = 'store_true'),
+           make_option('--update',
+                       help = 'like fold but only update the current patch files',
                        action = 'store_true')]
 
 
@@ -53,8 +58,10 @@ def func(parser, options, args):
     check_head_top_equal()
 
     commit_str = args[0]
+    commit_id = git_id(commit_str)
+    commit = git.Commit(commit_id)
 
-    if options.fold:
+    if options.fold or options.update:
         if not crt_series.get_current():
             raise CmdException, 'No patches applied'
     else:
@@ -64,17 +71,21 @@ def func(parser, options, args):
         elif len(patch_branch) == 2:
             patch = patch_branch[0]
         else:
-            raise CmdException, 'Unknown patch name'
+            patch = make_patch_name(commit.get_log())
+            if not patch:
+                raise CmdException, 'Unknown patch name'
 
-    commit_id = git_id(commit_str)
-    commit = git.Commit(commit_id)
+    if options.parent:
+        parent = git_id(options.parent)
+    else:
+        parent = commit.get_parent()
 
     if not options.reverse:
-        bottom = commit.get_parent()
+        bottom = parent
         top = commit_id
     else:
         bottom = commit_id
-        top = commit.get_parent()
+        top = parent
 
     if options.fold:
         print 'Folding commit %s...' % commit_id,
@@ -84,6 +95,18 @@ def func(parser, options, args):
         if not git.apply_diff(bottom, top):
             git.merge(bottom, git.get_head(), top)
 
+        print 'done'
+    elif options.update:
+        rev1 = git_id('//bottom')
+        rev2 = git_id('//top')
+        files = git.barefiles(rev1, rev2).split('\n')
+
+        print 'Updating with commit %s...' % commit_id,
+        sys.stdout.flush()
+
+        if not git.apply_diff(bottom, top, files = files):
+            raise CmdException, 'Patch updating failed'
+
         print 'done'
     else:
         message = commit.get_log()
