@@ -29,11 +29,13 @@ config = ConfigParser.RawConfigParser()
 config.add_section('stgit')
 config.set('stgit', 'autoresolved', 'no')
 config.set('stgit', 'smtpserver', 'localhost:25')
-config.set('stgit', 'smtpdelay', '2')
+config.set('stgit', 'smtpdelay', '5')
+config.set('stgit', 'pullcmd', 'git-pull')
 config.set('stgit', 'merger',
-           'diff3 -L local -L older -L remote -m -E ' \
+           'diff3 -L current -L ancestor -L patched -m -E ' \
            '"%(branch1)s" "%(ancestor)s" "%(branch2)s" > "%(output)s"')
 config.set('stgit', 'keeporig', 'yes')
+config.set('stgit', 'extensions', '.ancestor .current .patched')
 
 # Read the configuration files (if any) and override the default settings
 config.read('/etc/stgitrc')
@@ -49,3 +51,23 @@ if config.has_option('stgit', 'pager'):
 if config.has_section('gitmergeonefile'):
     for option, value in config.items('gitmergeonefile'):
         config.set('stgit', option, value)
+
+
+# cached extensions
+__extensions = None
+
+def file_extensions():
+    """Returns a dictionary with the conflict file extensions
+    """
+    global __extensions
+
+    if not __extensions:
+        cfg_ext = config.get('stgit', 'extensions').split()
+        if len(cfg_ext) != 3:
+            raise CmdException, '"extensions" configuration error'
+
+        __extensions = { 'ancestor': cfg_ext[0],
+                         'current':  cfg_ext[1],
+                         'patched':  cfg_ext[2] }
+
+    return __extensions
