@@ -21,7 +21,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 import sys, os
 
 from stgit.utils import *
-from stgit import git, basedir
+from stgit import git, basedir, templates
 from stgit.config import config
 
 
@@ -65,14 +65,14 @@ def __clean_comments(f):
     f.writelines(lines)
 
 def edit_file(series, line, comment, show_patch = True):
-    fname = '.stgit.msg'
-    tmpl = os.path.join(basedir.get(), 'patchdescr.tmpl')
+    fname = '.stgitmsg.txt'
+    tmpl = templates.get_template('patchdescr.tmpl')
 
     f = file(fname, 'w+')
     if line:
         print >> f, line
-    elif os.path.isfile(tmpl):
-        print >> f, file(tmpl).read().rstrip()
+    elif tmpl:
+        print >> f, tmpl,
     else:
         print >> f
     print >> f, __comment_prefix, comment
@@ -128,6 +128,8 @@ class Patch:
         self.__dir = os.path.join(self.__series_dir, self.__name)
         self.__refs_dir = refs_dir
         self.__top_ref_file = os.path.join(self.__refs_dir, self.__name)
+        self.__log_ref_file = os.path.join(self.__refs_dir,
+                                           self.__name + '.log')
 
     def create(self):
         os.mkdir(self.__dir)
@@ -139,23 +141,33 @@ class Patch:
             os.remove(os.path.join(self.__dir, f))
         os.rmdir(self.__dir)
         os.remove(self.__top_ref_file)
+        if os.path.exists(self.__log_ref_file):
+            os.remove(self.__log_ref_file)
 
     def get_name(self):
         return self.__name
 
     def rename(self, newname):
         olddir = self.__dir
-        old_ref_file = self.__top_ref_file
+        old_top_ref_file = self.__top_ref_file
+        old_log_ref_file = self.__log_ref_file
         self.__name = newname
         self.__dir = os.path.join(self.__series_dir, self.__name)
         self.__top_ref_file = os.path.join(self.__refs_dir, self.__name)
+        self.__log_ref_file = os.path.join(self.__refs_dir,
+                                           self.__name + '.log')
 
         os.rename(olddir, self.__dir)
-        os.rename(old_ref_file, self.__top_ref_file)
+        os.rename(old_top_ref_file, self.__top_ref_file)
+        if os.path.exists(old_log_ref_file):
+            os.rename(old_log_ref_file, self.__log_ref_file)
 
     def __update_top_ref(self, ref):
         write_string(self.__top_ref_file, ref)
 
+    def __update_log_ref(self, ref):
+        write_string(self.__log_ref_file, ref)
+
     def update_top_ref(self):
         top = self.get_top()
         if top:
@@ -274,6 +286,13 @@ class Patch:
                 address = os.environ['GIT_COMMITTER_EMAIL']
         self.__set_field('commemail', address)
 
+    def get_log(self):
+        return self.__get_field('log')
+
+    def set_log(self, value, backup = False):
+        self.__set_field('log', value)
+        self.__update_log_ref(value)
+
 
 class Series:
     """Class including the operations on series
@@ -426,17 +445,24 @@ class Series:
         """
         return os.path.isdir(self.__patch_dir)
 
-    def init(self):
+    def init(self, create_at=False):
         """Initialises the stgit series
         """
         bases_dir = os.path.join(self.__base_dir, 'refs', 'bases')
 
-        if self.is_initialised():
+        if os.path.exists(self.__patch_dir):
             raise StackException, self.__patch_dir + ' already exists'
+        if os.path.exists(self.__refs_dir):
+            raise StackException, self.__refs_dir + ' already exists'
+        if os.path.exists(self.__base_file):
+            raise StackException, self.__base_file + ' already exists'
+
+        if (create_at!=False):
+            git.create_branch(self.__name, create_at)
+
         os.makedirs(self.__patch_dir)
 
-        if not os.path.isdir(bases_dir):
-            os.makedirs(bases_dir)
+        create_dirs(bases_dir)
 
         create_empty_file(self.__applied_file)
         create_empty_file(self.__unapplied_file)
@@ -494,9 +520,14 @@ class Series:
         git.rename_branch(self.__name, to_name)
 
         if os.path.isdir(self.__series_dir):
-            os.rename(self.__series_dir, to_stack.__series_dir)
+            rename(os.path.join(self.__base_dir, 'patches'),
+                   self.__name, to_stack.__name)
         if os.path.exists(self.__base_file):
-            os.rename(self.__base_file, to_stack.__base_file)
+            rename(os.path.join(self.__base_dir, 'refs', 'bases'),
+                   self.__name, to_stack.__name)
+        if os.path.exists(self.__refs_dir):
+            rename(os.path.join(self.__base_dir, 'refs', 'patches'),
+                   self.__name, to_stack.__name)
 
         self.__init__(to_name)
 
@@ -504,8 +535,7 @@ class Series:
         """Clones a series
         """
         base = read_string(self.get_base_file())
-        git.create_branch(target_series, tree_id = base)
-        Series(target_series).init()
+        Series(target_series).init(create_at = base)
         new_series = Series(target_series)
 
         # generate an artificial description file
@@ -551,16 +581,19 @@ class Series:
             else:
                 print 'Patch directory %s is not empty.' % self.__name
             if not os.listdir(self.__series_dir):
-                os.rmdir(self.__series_dir)
+                remove_dirs(os.path.join(self.__base_dir, 'patches'),
+                            self.__name)
             else:
                 print 'Series directory %s is not empty.' % self.__name
             if not os.listdir(self.__refs_dir):
-                os.rmdir(self.__refs_dir)
+                remove_dirs(os.path.join(self.__base_dir, 'refs', 'patches'),
+                            self.__name)
             else:
                 print 'Refs directory %s is not empty.' % self.__refs_dir
 
         if os.path.exists(self.__base_file):
-            os.remove(self.__base_file)
+            remove_file_and_dirs(
+                os.path.join(self.__base_dir, 'refs', 'bases'), self.__name)
 
     def refresh_patch(self, files = None, message = None, edit = False,
                       show_patch = False,
@@ -568,7 +601,7 @@ class Series:
                       author_name = None, author_email = None,
                       author_date = None,
                       committer_name = None, committer_email = None,
-                      backup = False):
+                      backup = False, sign_str = None, log = 'refresh'):
         """Generates a new commit for the given patch
         """
         name = self.get_current()
@@ -600,6 +633,10 @@ class Series:
         if not committer_email:
             committer_email = patch.get_commemail()
 
+        if sign_str:
+            descr = '%s\n%s: %s <%s>\n' % (descr.rstrip(), sign_str,
+                                           committer_name, committer_email)
+
         bottom = patch.get_bottom()
 
         commit_id = git.commit(files = files,
@@ -621,6 +658,9 @@ class Series:
         patch.set_commname(committer_name)
         patch.set_commemail(committer_email)
 
+        if log:
+            self.log_patch(patch, log)
+
         return commit_id
 
     def undo_refresh(self):
@@ -640,7 +680,8 @@ class Series:
             raise StackException, 'No refresh undo information available'
 
         git.reset(tree_id = old_top, check_out = False)
-        patch.restore_old_boundaries()
+        if patch.restore_old_boundaries():
+            self.log_patch(patch, 'undo')
 
     def new_patch(self, name, message = None, can_edit = True,
                   unapplied = False, show_patch = False,
@@ -684,19 +725,24 @@ class Series:
         patch.set_commemail(committer_email)
 
         if unapplied:
+            self.log_patch(patch, 'new')
+
             patches = [patch.get_name()] + self.get_unapplied()
 
             f = file(self.__unapplied_file, 'w+')
             f.writelines([line + '\n' for line in patches])
             f.close()
-        else:
-            if before_existing:
-                insert_string(self.__applied_file, patch.get_name())
-                if not self.get_current():
-                    self.__set_current(name)
-            else:
-                append_string(self.__applied_file, patch.get_name())
+        elif before_existing:
+            self.log_patch(patch, 'new')
+
+            insert_string(self.__applied_file, patch.get_name())
+            if not self.get_current():
                 self.__set_current(name)
+        else:
+            append_string(self.__applied_file, patch.get_name())
+            self.__set_current(name)
+
+            self.refresh_patch(cache_update = False, log = 'new')
 
     def delete_patch(self, name):
         """Deletes a patch
@@ -743,7 +789,8 @@ class Series:
 
             # top != bottom always since we have a commit for each patch
             if head == bottom:
-                # reset the backup information
+                # reset the backup information. No logging since the
+                # patch hasn't changed
                 patch.set_bottom(head, backup = True)
                 patch.set_top(top, backup = True)
 
@@ -774,6 +821,8 @@ class Series:
 
                     patch.set_bottom(head, backup = True)
                     patch.set_top(top, backup = True)
+
+                    self.log_patch(patch, 'push(f)')
                 else:
                     top = head
                     # stop the fast-forwarding, must do a real merge
@@ -809,7 +858,7 @@ class Series:
 
         merged = []
         for p in patches:
-            if git.apply_diff(p.get_top(), p.get_bottom(), False):
+            if git.apply_diff(p.get_top(), p.get_bottom()):
                 merged.append(p.get_name())
         merged.reverse()
 
@@ -844,7 +893,7 @@ class Series:
             patch.set_top(head, backup = True)
             modified = True
         elif head == bottom:
-            # reset the backup information
+            # reset the backup information. No need for logging
             patch.set_bottom(bottom, backup = True)
             patch.set_top(top, backup = True)
 
@@ -884,7 +933,11 @@ class Series:
             if not ex:
                 # if the merge was OK and no conflicts, just refresh the patch
                 # The GIT cache was already updated by the merge operation
-                self.refresh_patch(cache_update = False)
+                if modified:
+                    log = 'push(m)'
+                else:
+                    log = 'push'
+                self.refresh_patch(cache_update = False, log = log)
             else:
                 raise StackException, str(ex)
 
@@ -907,9 +960,13 @@ class Series:
 
         git.reset()
         self.pop_patch(name)
-        return patch.restore_old_boundaries()
+        ret = patch.restore_old_boundaries()
+        if ret:
+            self.log_patch(patch, 'undo')
 
-    def pop_patch(self, name):
+        return ret
+
+    def pop_patch(self, name, keep = False):
         """Pops the top patch from the stack
         """
         applied = self.get_applied()
@@ -918,7 +975,12 @@ class Series:
 
         patch = Patch(name, self.__patch_dir, self.__refs_dir)
 
-        git.switch(patch.get_bottom())
+        # only keep the local changes
+        if keep and not git.apply_diff(git.get_head(), patch.get_bottom()):
+            raise StackException, \
+                  'Failed to pop patches while preserving the local changes'
+
+        git.switch(patch.get_bottom(), keep)
 
         # save the new applied list
         idx = applied.index(name) + 1
@@ -989,3 +1051,20 @@ class Series:
             f.close()
         else:
             raise StackException, 'Unknown patch "%s"' % oldname
+
+    def log_patch(self, patch, message):
+        """Generate a log commit for a patch
+        """
+        top = git.get_commit(patch.get_top())
+        msg = '%s\t%s' % (message, top.get_id_hash())
+
+        old_log = patch.get_log()
+        if old_log:
+            parents = [old_log]
+        else:
+            parents = []
+
+        log = git.commit(message = msg, parents = parents,
+                         cache_update = False, tree_id = top.get_tree(),
+                         allowempty = True)
+        patch.set_log(log)
